{% extends 'hiddify-flask-admin/list.html' %}

{% block body %}
{{super()}}

<div class="callout callout-info">
    {{_('domain.intro')}}
</div>


<style>
    .table td {
        vertical-align: middle;
    }

    td.col-domain {
        /* direction:ltr; */
    }
</style>
{% endblock %}